// ***************************************************************************
// ***************************************************************************
// Rejeesh.Kutty@Analog.com (c) Analog Devices Inc.
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xparameters.h"
#include "xuartlite_l.h"
#include "cf_adv7x11_demo.h"

#define CFV_BASEADDR        XPAR_AXI_HDMI_16B_ES_0_BASEADDR
#define CFA_BASEADDR        XPAR_AXI_SPDIF_TX_0_BASEADDR
#define CF_CLKGEN_BASEADDR  XPAR_AXI_CLKGEN_0_BASEADDR
#define DDR_BASEADDR        XPAR_DDR3_SDRAM_S_AXI_BASEADDR
#define VDMA_BASEADDR       XPAR_AXI_VDMA_0_BASEADDR
#define ADMA_BASEADDR       XPAR_AXI_DMA_0_BASEADDR
#define IIC_BASEADDR        XPAR_AXI_IIC_0_BASEADDR
#define UART_BASEADDR       XPAR_RS232_UART_1_BASEADDR

#define H_TOTAL             2200
#define H_ACTIVE            1920
#define H_STRIDE            1920
#define V_TOTAL             1125
#define V_ACTIVE            1080
#define A_SAMPLE_FREQ       48000
#define A_FREQ              1400

#define VIDEO_LENGTH  (H_ACTIVE*V_ACTIVE)
#define AUDIO_LENGTH  (A_SAMPLE_FREQ/A_FREQ)
#define VIDEO_BASEADDR  DDR_BASEADDR + 0x2000000
#define AUDIO_BASEADDR  DDR_BASEADDR + 0x1000000

// ***************************************************************************
// ***************************************************************************

extern char inbyte(void);
void xil_printf(const char *ctrl1, ...);

void delay_ms(u32 ms_count) {
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

u32 user_exit(void) {
  while (!XUartLite_IsReceiveEmpty(UART_BASEADDR)) {
    if (XUartLite_RecvByte(UART_BASEADDR) == 'q') {
      return(1);
    }
  }
  return(0);
}

void iic_select(u32 sel) {
  u32 rdata;
  Xil_Out32((IIC_BASEADDR + 0x124), 0x000); // reset iic mux
  Xil_Out32((IIC_BASEADDR + 0x124), 0x001); // reset iic mux
  Xil_Out32((IIC_BASEADDR + 0x100), 0x002); // reset tx fifo
  Xil_Out32((IIC_BASEADDR + 0x100), 0x001); // enable iic
  Xil_Out32((IIC_BASEADDR + 0x108), 0x1e0); // mux select
  Xil_Out32((IIC_BASEADDR + 0x108), (0x200 | sel)); // enable selected device
  while ((Xil_In32(IIC_BASEADDR + 0x104) & 0x80) == 0x00) {delay_ms(1);}
  delay_ms(10);
  Xil_Out32((IIC_BASEADDR + 0x100), 0x002); // reset tx fifo
  Xil_Out32((IIC_BASEADDR + 0x100), 0x001); // enable iic
  Xil_Out32((IIC_BASEADDR + 0x108), 0x1e1); // mux select
  Xil_Out32((IIC_BASEADDR + 0x108), 0x201); // read back data
  while ((Xil_In32(IIC_BASEADDR + 0x104) & 0x40) == 0x40) {delay_ms(1);}
  rdata = Xil_In32(IIC_BASEADDR + 0x10c) & 0xff;
  xil_printf("iic_select: %02x\n\r", rdata);
}

u32 iic_read(u32 sel, u32 addr, u32 display) {
  u32 rdata;
  Xil_Out32((IIC_BASEADDR + 0x100), 0x002); // reset tx fifo
  Xil_Out32((IIC_BASEADDR + 0x100), 0x001); // enable iic
  Xil_Out32((IIC_BASEADDR + 0x108), (0x100 | sel)); // adv7x11 select
  Xil_Out32((IIC_BASEADDR + 0x108), addr); // adv7x11 address
  Xil_Out32((IIC_BASEADDR + 0x108), (0x101 | sel)); // adv7x11 select
  Xil_Out32((IIC_BASEADDR + 0x108), 0x201); // adv7x11 data
  while ((Xil_In32(IIC_BASEADDR + 0x104) & 0x40) == 0x40) {delay_ms(1);}
  delay_ms(10);
  rdata = Xil_In32(IIC_BASEADDR + 0x10c) & 0xff;
  if (display == 1) {
    xil_printf("iic_read: sel(%02x), addr(%02x) data(%02x)\n\r", sel, addr, rdata);
  }
  return(rdata);
}

void iic_write(u32 sel, u32 addr, u32 data) {
  Xil_Out32((IIC_BASEADDR + 0x100), 0x002); // reset tx fifo
  Xil_Out32((IIC_BASEADDR + 0x100), 0x001); // enable iic
  Xil_Out32((IIC_BASEADDR + 0x108), (0x100 | sel)); // adv7x11 select
  Xil_Out32((IIC_BASEADDR + 0x108), addr); // adv7x11 address
  Xil_Out32((IIC_BASEADDR + 0x108), (0x200 | data)); // adv7x11 data
  while ((Xil_In32(IIC_BASEADDR + 0x104) & 0x80) == 0x00) {delay_ms(1);}
  delay_ms(10);
}

void ddr_video_wr() {

  u32 n;
  u32 d;
  u32 dcnt;

  dcnt = 0;
  xil_printf("DDR write: started (length %d)\n\r", IMG_LENGTH);
  for (n = 0; n < IMG_LENGTH; n++) {
    for (d = 0; d < ((IMG_DATA[n]>>24) & 0xff); d++) {
      Xil_Out32((VIDEO_BASEADDR+(dcnt*4)), (IMG_DATA[n] & 0xffffff));
      dcnt = dcnt + 1;
    }
  }
  xil_printf("DDR write: completed (total %d)\n\r", dcnt);
}

void ddr_audio_wr() {

  u32 n;
  u32 scnt;
  u32 sincr;

  xil_printf("DDR audio write: started\n\r");
  scnt = 0;
  sincr = (65536*2)/AUDIO_LENGTH;
  for (n = 0; n < 32; n++) {
	  Xil_Out32((AUDIO_BASEADDR+(n*4)), 0x00); // init descriptors
  }
  Xil_Out32((AUDIO_BASEADDR+0x00), (AUDIO_BASEADDR + 0x40)); // next descriptor
  Xil_Out32((AUDIO_BASEADDR+0x08), (AUDIO_BASEADDR + 0x80)); // start address
  Xil_Out32((AUDIO_BASEADDR+0x40), (AUDIO_BASEADDR + 0x00)); // next descriptor
  Xil_Out32((AUDIO_BASEADDR+0x48), (AUDIO_BASEADDR + 0x80)); // start address
  Xil_Out32((AUDIO_BASEADDR+0x18), (0x8000000 | (AUDIO_LENGTH*8))); // no. of bytes
  Xil_Out32((AUDIO_BASEADDR+0x58), (0x4000000 | (AUDIO_LENGTH*8))); // no. of bytes
  Xil_Out32((AUDIO_BASEADDR+0x1c), 0x00); // status
  Xil_Out32((AUDIO_BASEADDR+0x5c), 0x00); // status
  for (n = 0; n < AUDIO_LENGTH; n++) {
    Xil_Out32((AUDIO_BASEADDR+0x80+(n*4)), ((scnt << 16) | scnt));
    scnt = (n > (AUDIO_LENGTH/2)) ? (scnt-sincr) : (scnt+sincr);
  }
  xil_printf("DDR audio write: completed (total %d)\n\r", AUDIO_LENGTH);
}

// ***************************************************************************
// ***************************************************************************

int main() {

  u32 data;

  init_platform();
  ddr_video_wr();
  ddr_audio_wr();

  delay_ms(1);
  data = Xil_In32(CF_CLKGEN_BASEADDR + (0x1f*4));
  if ((data & 0x1) == 0x0) {
    xil_printf("CLKGEN (148.5MHz) out of lock (0x%04x)\n\r", data);
    return(0);
  }

  iic_select(0xf);
  iic_write(0x40, 0x03, 0xea); // configure direction
  iic_write(0x40, 0x01, 0x05); // configure data

  // transmit (v2h) parameters

  Xil_Out32((VDMA_BASEADDR + 0x000), 0x3); // enable circular mode
  Xil_Out32((VDMA_BASEADDR + 0x05c), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x060), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x064), VIDEO_BASEADDR); // start address
  Xil_Out32((VDMA_BASEADDR + 0x058), (H_STRIDE*4)); // h offset (2048 * 4) bytes
  Xil_Out32((VDMA_BASEADDR + 0x054), (H_ACTIVE*4)); // h size (1920 * 4) bytes
  Xil_Out32((VDMA_BASEADDR + 0x050), V_ACTIVE); // v size (1080)

  Xil_Out32((CFV_BASEADDR + 0x08), ((H_ACTIVE << 16) | H_TOTAL)); // v2h, h-de parameters
  Xil_Out32((CFV_BASEADDR + 0x0c), ((V_ACTIVE << 16) | V_TOTAL)); // v2h, v-de parameters
  Xil_Out32((CFV_BASEADDR + 0x04), 0x1); // v2h, enable, csc-bypass & tpg

  Xil_Out32((CFA_BASEADDR + 0x04), 0x040); // sample frequency
  Xil_Out32((CFA_BASEADDR + 0x00), 0xc03); // clock ratio, data enable & signal enable

  // wait for hpd

  while ((iic_read(0x72, 0x96, 0x01) & 0x80) != 0x80) {
    delay_ms(1);
  }

  iic_write(0x98, 0xf4, 0x80); // CEC
  iic_write(0x98, 0xf5, 0x7c); // INFOFRAME
  iic_write(0x98, 0xf8, 0x4c); // DPLL
  iic_write(0x98, 0xf9, 0x64); // KSV
  iic_write(0x98, 0xfa, 0x6c); // EDID
  iic_write(0x98, 0xfb, 0x68); // HDMI
  iic_write(0x98, 0xfd, 0x44); // CP
  iic_write(0x98, 0x01, 0x06); // Prim_Mode =110b HDMI-GR
  iic_write(0x98, 0x02, 0xf5); // Auto CSC, YCrCb out, Set op_656 bit
  iic_write(0x98, 0x03, 0x80); // 16-Bit SDR ITU-R BT.656 4:2:2 Mode 0
  iic_write(0x98, 0x04, 0x62); // OP_CH_SEL[2:0] = 011b - (P[15:8] Y, P[7:0] CrCb), XTAL_FREQ[1:0] = 01b (28.63636 MHz)
  iic_write(0x98, 0x05, 0x2c); // AV Codes on
  iic_write(0x44, 0x7b, 0x05); // 
  iic_write(0x98, 0x0b, 0x44); // Power up part
  iic_write(0x98, 0x0c, 0x42); // Power up part
  iic_write(0x98, 0x14, 0x7f); // Max Drive Strength
  iic_write(0x98, 0x15, 0x80); // Disable Tristate of Pins
  iic_write(0x98, 0x19, 0x83); // LLC DLL phase
  iic_write(0x98, 0x33, 0x40); // LLC DLL enable
  iic_write(0x44, 0xba, 0x01); // Set HDMI FreeRun
  iic_write(0x64, 0x40, 0x81); // Disable HDCP 1.1 features
  iic_write(0x68, 0x9b, 0x03); // ADI recommended setting
  iic_write(0x68, 0xc1, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc2, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc3, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc4, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc5, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc6, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc7, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc8, 0x01); // ADI recommended setting
  iic_write(0x68, 0xc9, 0x01); // ADI recommended setting
  iic_write(0x68, 0xca, 0x01); // ADI recommended setting
  iic_write(0x68, 0xcb, 0x01); // ADI recommended setting
  iic_write(0x68, 0xcc, 0x01); // ADI recommended setting
  iic_write(0x68, 0x00, 0x08); // Set HDMI Input Port A (BG_MEAS_PORT_SEL = 001b)
  iic_write(0x68, 0x02, 0x03); // Enable Ports A & B in background mode
  iic_write(0x68, 0x83, 0xfc); // Enable clock terminators for port A & B
  iic_write(0x68, 0x6f, 0x0c); // ADI recommended setting
  iic_write(0x68, 0x85, 0x1f); // ADI recommended setting
  iic_write(0x68, 0x87, 0x70); // ADI recommended setting
  iic_write(0x68, 0x8d, 0x04); // LFG Port A
  iic_write(0x68, 0x8e, 0x1e); // HFG Port A
  iic_write(0x68, 0x1a, 0x8a); // unmute audio
  iic_write(0x68, 0x57, 0xda); // ADI recommended setting
  iic_write(0x68, 0x58, 0x01); // ADI recommended setting
  iic_write(0x68, 0x75, 0x10); // DDC drive strength
  iic_write(0x68, 0x90, 0x04); // LFG Port B
  iic_write(0x68, 0x91, 0x1e); // HFG Port B

  // disable csc

  iic_write(0x44, 0x69, 0x10);
  iic_write(0x44, 0x68, 0x00);
  iic_write(0x44, 0xbd, 0x10);
  iic_write(0x98, 0x04, 0x62);

  iic_write(0x72, 0x01, 0x00); // Set N Value(6144)
  iic_write(0x72, 0x02, 0x18); // Set N Value(6144)
  iic_write(0x72, 0x03, 0x00); // Set N Value(6144)
  iic_write(0x72, 0x15, 0x02); // Input YCbCr 4:2:2 with embedded syncs
  iic_write(0x72, 0x16, 0xb9); // Output format 422, 16-bit input
  iic_write(0x72, 0x18, 0x46); // CSC disabled
  iic_write(0x72, 0x40, 0x80); // General Control packet enable
  iic_write(0x72, 0x41, 0x10); // Power down control
  iic_write(0x72, 0x48, 0x10); // Data left justified
  iic_write(0x72, 0x49, 0xa8);

  iic_write(0x72, 0x4c, 0x00); // 8 bit Output
  iic_write(0x72, 0x55, 0x20); // Set YCrCb 444 in AVinfo Frame
  iic_write(0x72, 0x56, 0x08); // Set active format Aspect
  iic_write(0x72, 0x96, 0x20); // HPD Interrupt clear
  iic_write(0x72, 0x98, 0x03); // ADI Recommended Write
  iic_write(0x72, 0x99, 0x02); // ADI Recommended Write
  iic_write(0x72, 0x9a, 0xe0); // ADI Recommended Write
  iic_write(0x72, 0x9c, 0x30); // PLL Filter R1 Value
  iic_write(0x72, 0x9d, 0x61); // Set clock divide
  iic_write(0x72, 0xa2, 0xa4); // ADI Recommended Write
  iic_write(0x72, 0xa3, 0xa4); // ADI Recommended Write
  iic_write(0x72, 0xa5, 0x44); // ADI Recommended Write
  iic_write(0x72, 0xab, 0x40); // ADI Recommended Write
  iic_write(0x72, 0xaf, 0x06); // Set HDMI Mode
  iic_write(0x72, 0xba, 0x00); // No clock delay
  iic_write(0x72, 0xd0, 0x3c); // No clock delay
  iic_write(0x72, 0xd1, 0xff); // ADI Recommended Write
  iic_write(0x72, 0xde, 0x9c); // ADI Recommended Write
  iic_write(0x72, 0xe0, 0xd0); // ADI Recommended Write
  iic_write(0x72, 0xe4, 0x60); // VCO_Swing_Reference_Voltage
  iic_write(0x72, 0xf9, 0x00); // VCO_Swing_Reference_Voltage
  iic_write(0x72, 0xfa, 0x00); // Nbr of times to search for good phase
  iic_write(0x72, 0x30, 0x16);
  iic_write(0x72, 0x31, 0x02);
  iic_write(0x72, 0x32, 0xc0);
  iic_write(0x72, 0x33, 0x10);
  iic_write(0x72, 0x34, 0x05);
  iic_write(0x72, 0x17, 0x02);

  iic_write(0x72, 0x0a, 0x10);
  iic_write(0x72, 0x0b, 0x8e);
  iic_write(0x72, 0x0c, 0x00);
  iic_write(0x72, 0x73, 0x01);
  iic_write(0x72, 0x14, 0x02);

  iic_read(0x72, 0x42, 1);
  iic_read(0x72, 0xc8, 1);
  iic_read(0x72, 0x9e, 1);
  iic_read(0x72, 0x96, 1);
  iic_read(0x72, 0x3e, 1);
  iic_read(0x72, 0x3d, 1);
  iic_read(0x72, 0x3c, 1);
  iic_read(0x44, 0xf4, 1);
  iic_read(0x44, 0xbd, 1);
  iic_read(0x68, 0x1d, 1);
  iic_read(0x98, 0xe0, 1);

  xil_printf("Generating audio clicks (press 'q' to exit).\n\r");
  while (user_exit() == 0) {
    Xil_Out32((AUDIO_BASEADDR+0x1c), 0x00); // status
    Xil_Out32((AUDIO_BASEADDR+0x5c), 0x00); // status
    Xil_Out32((ADMA_BASEADDR + 0x00), 0); // clear dma operations
    Xil_Out32((ADMA_BASEADDR + 0x08), AUDIO_BASEADDR); // head descr.
	  Xil_Out32((ADMA_BASEADDR + 0x00), 1); // enable dma operations
    Xil_Out32((ADMA_BASEADDR + 0x10), (AUDIO_BASEADDR+0x40)); // tail descr.
    delay_ms(100);
  }

  xil_printf("done.\n\r");
  cleanup_platform();
  return(0);
}

// ***************************************************************************
// ***************************************************************************

