//--------------------------
//- spdif.c                - 
//-                        -
//-	Created on: 21.11.2011 -
//-	    Author: Alex Tofan -
//--------------------------

#include "spdif.h"
#include "xil_io.h"
#include "xparameters.h"
//----------------------------------------------------------------------------------
//--- Write functions --------------------------------------------------------------
//----------------------------------------------------------------------------------

// Config Register
void W_SPDIF_CONFIG(int value)
{
    Xil_Out32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_CONFIG, value);
}
// Channel Status Register
void W_SPDIF_CH_STAT(int value)
{
    Xil_Out32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_CH_STAT, value);
}
// Interrupt Mask
void W_SPDIF_INT_MASK(int value)
{  
    Xil_Out32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_INT_MASK, value);
}
// Interrupt Status
void W_SPDIF_INT_STAT(int value)
{
    Xil_Out32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_INT_STAT, value);
}

//----------------------------------------------------------------------------------
//--- Write functions --------------------------------------------------------------
//----------------------------------------------------------------------------------

// Config Register
int R_SPDIF_CONFIG(void)
{
    return(Xil_In32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_CONFIG));
}
// Channel Status Register
int R_SPDIF_CH_STAT(void)
{
    return(Xil_In32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_CH_STAT));
}
// Interrupt Mask
int R_SPDIF_INT_MASK(void)
{  
    return(Xil_In32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_INT_MASK));
}
// Interrupt Status
int R_SPDIF_INT_STAT(void)
{
    return(Xil_In32(XPAR_AXI_SPDIF_TX_0_BASEADDR + SPDIF_TX_INT_STAT));
}

void SPDIF_INIT(void)
{
	int CfgValue;
	//-----------------------------------------------------------------------------------------------
	//--- SPDIF Core Configuration ------------------------------------------------------------------
	//-----------------------------------------------------------------------------------------------
	//------------------------------------------------------------
	// Config Register                                           -
	// Modify Values below according to datasheet specifications -
	//------------------------------------------------------------
	//CfgValue = R_SPDIF_CONFIG();
	CfgValue = 0;
	CfgValue |= (0 << SPDIF_MODE)       | //MODE_16
				(12 << SPDIF_RATIO) 	    |//12
				(0 << SPDIF_UDATEN) 	|
				(0 << SPDIF_CHSTEN) 	|
				(0 << SPDIF_TINTEN) 	|
				(0 << SPDIF_TXDATA) 	|
				(0 << SPDIF_TXEN);
	W_SPDIF_CONFIG(CfgValue);
	//------------------------------------------------------------

	//------------------------------------------------------------
	// Channel Status Register                                   -
	// Modify Values below according to datasheet specifications -
	//------------------------------------------------------------
	//CfgValue = R_SPDIF_CH_STAT();
	CfgValue = 0;
	CfgValue |= (1 << SPDIF_FREQ) 	|//1
				(0 << SPDIF_GSTAT)	|
				(0 << SPDIF_PREEM) 	|
				(0 << SPDIF_COPY) 	|
				(0 << SPDIF_AUDIO);
	W_SPDIF_CH_STAT(CfgValue);
	//------------------------------------------------------------

	//------------------------------------------------------------
	// Interrupt Mask Register                                   -
	// Modify Values below according to datasheet specifications -
	//------------------------------------------------------------
	//CfgValue = R_SPDIF_INT_MASK();
	CfgValue = 0;
	CfgValue |= (0 << SPDIF_HCSBF_MSK) 	|
				(0 << SPDIF_LCSBF_MSK)	|
				(0 << SPDIF_HSBF_MSK) 	|
				(0 << SPDIF_LSBF_MSK);
	W_SPDIF_INT_MASK(CfgValue);
	//------------------------------------------------------------

	//------------------------------------------------------------
	// Interrupt Status Register                                 -
	// Modify Values below according to datasheet specifications -
	//------------------------------------------------------------
	//CfgValue = R_SPDIF_INT_STAT();
	CfgValue = 0;
	CfgValue |= (0 << SPDIF_HCSBF_INT) 	|
				(0 << SPDIF_LCSBF_INT)	|
				(0 << SPDIF_HSBF_INT) 	|
				(0 << SPDIF_LSBF_INT);
	W_SPDIF_INT_STAT(CfgValue);
	//------------------------------------------------------------
}

void SPDIF_START(void)
{
	int CfgValue;
	CfgValue = R_SPDIF_CONFIG();
	CfgValue |= (1 << SPDIF_TXEN) 	|
				(1 << SPDIF_TXDATA);
	W_SPDIF_CONFIG(CfgValue);
}

void SPDIF_STOP(void)
{
	int CfgValue;
	CfgValue = R_SPDIF_CONFIG();
	CfgValue &= ~((1 << SPDIF_TXEN) |
				  (1 << SPDIF_TXDATA));
	W_SPDIF_CONFIG(CfgValue);
}


